#include <stdint.h>
#include "pstorage.h"

#include "nrf_drv_wdt.h"

#include "app_error.h"

static pstorage_handle_t       			base_handle;
static pstorage_handle_t				block_1_handle;
static pstorage_handle_t				block_2_handle;

static uint8_t pstorage_wait_flag = 0;
static pstorage_block_t pstorage_wait_handle = 0;

static void pstorage_cb_handler(pstorage_handle_t  * handle,
                               uint8_t              op_code,
                               uint32_t             result,
                               uint8_t            * p_data,
                               uint32_t             data_len)
{	
	if(handle->block_id == pstorage_wait_handle)
	{
		pstorage_wait_flag = 0;
	}

	switch(op_code)
	{
		case PSTORAGE_STORE_OP_CODE:
			if(result != NRF_SUCCESS)	APP_ERROR_CHECK(result);
			break;
		case PSTORAGE_LOAD_OP_CODE:
			if(result != NRF_SUCCESS)	APP_ERROR_CHECK(result);
			break;
		case PSTORAGE_CLEAR_OP_CODE:
			if(result != NRF_SUCCESS)	APP_ERROR_CHECK(result);
			break;
		case PSTORAGE_UPDATE_OP_CODE:
			if(result != NRF_SUCCESS)	APP_ERROR_CHECK(result);
			break;
	}

}

void pstorage_custom_init(void)
{
	uint8_t init_data[4] = {0,0,0,0};
	pstorage_module_param_t param;
	uint32_t                retval;
				
	param.block_size  = 16;
	param.block_count = 4;
	param.cb          = pstorage_cb_handler;
			
	retval = pstorage_register(&param, &base_handle);
	if (retval != NRF_SUCCESS)
	{
			APP_ERROR_CHECK(retval);
	}
	
	// Request to get identifier for 1st block. 
	retval = pstorage_block_identifier_get(&base_handle, 0, &block_1_handle);
	if (retval != NRF_SUCCESS)
	{
			APP_ERROR_CHECK(retval);
	}
	
	// Request to get identifier for 2nd block. 
	retval = pstorage_block_identifier_get(&base_handle, 1, &block_2_handle);
	if (retval != NRF_SUCCESS)
	{
			APP_ERROR_CHECK(retval);
	}
	
	retval = pstorage_store(&block_1_handle, init_data, 4, 0);
	if (retval == NRF_SUCCESS)
	{
		APP_ERROR_CHECK(retval);
	}

	pstorage_wait_handle = block_2_handle.block_id;
	pstorage_wait_flag = 1;
	
	retval = pstorage_store(&block_2_handle, init_data, 4, 0);
	if (retval != NRF_SUCCESS)
	{
		APP_ERROR_CHECK(retval);
	}
	
	while(pstorage_wait_flag) { nrf_drv_wdt_feed(); }
}

void pstorage_custom_read(pstorage_custom_t block, uint32_t * data)
{
	uint8_t data_array[4];
	pstorage_handle_t read_handle;
	
	switch(block)
	{
		case block_1:
			read_handle = block_1_handle;
			break;
		case block_2:
			read_handle = block_2_handle;
			break;
	}
	
	pstorage_wait_handle = read_handle.block_id;
	pstorage_wait_flag = 1;
	
	pstorage_load(data_array, &read_handle, 4, 0);
	
	while(pstorage_wait_flag) { nrf_drv_wdt_feed(); }
	
  *data = data_array[0] | (data_array[1] << 8) | (data_array[2] << 16) | (data_array[3] << 24);
}

void pstorage_custom_clear(void)
{
	uint32_t retval;
	
	pstorage_wait_handle = base_handle.block_id;
	pstorage_wait_flag = 1;
	
	// Request clearing of all blocks in the module.
	retval = pstorage_clear(&base_handle, 4 * 16);

	if (retval == NRF_SUCCESS)
	{
			while(pstorage_wait_flag) { nrf_drv_wdt_feed(); }
	}
	else
	{
			APP_ERROR_CHECK(retval);
	}
}

void pstorage_custom_update(pstorage_custom_t block, uint32_t data)
{
	uint32_t retval;
	uint8_t update_array[4];
	pstorage_handle_t update_handle;
	
	switch(block)
	{
		case block_1:
			update_handle = block_1_handle;
			break;
		case block_2:
			update_handle = block_2_handle;
			break;
	}
	
	update_array[0] = (data & 0xFF000000) >> 24;
	update_array[1] = (data & 0x00FF0000) >> 16;
	update_array[2] = (data & 0x0000FF00) >> 8;
	update_array[3] = (data & 0x000000FF);

	pstorage_wait_handle = update_handle.block_id;
	pstorage_wait_flag = 1;	
	
	retval = pstorage_update(&update_handle, update_array, 4, 0);
	
		if (retval == NRF_SUCCESS)
	{
			while(pstorage_wait_flag) { nrf_drv_wdt_feed(); }
	}
	else
	{
			APP_ERROR_CHECK(retval);
	}
}